module Arke
  module Resource
    module Callbacks
      extend ActiveSupport::Concern

      included do
        class_attribute    :after_initialize_callbacks

        alias_method_chain :initialize, :after_initializers

      end


      module ClassMethods

        def after_initialize(method=nil, &block)
          self.after_initializers ||= []
          if block_given?
            self.after_initializers << block
          else
            self.after_initializers << method
          end
        end

      end

      private

      def initialize_with_after_initializers(attributes={})
        initialize_without_after_initializers attributes
        self.class.after_initialize_callbacks ||= []
        self.class.after_initialize_callbacks.each do |callback|
          execute_callback(callback, attributes)
        end
      end

      def execute_callback(callback, *args)
        if callback.is_a? Symbol
          self.send(callback, *args)
        else
          callback.call(*args)
        end
      end


    end
  end
end