module Arke
  module Resource
    class Collection
      include Enumerable

      attr_reader :parent
      delegate :delete_all, to: :parent

      def initialize(parent, array)
        raise Errors::UnexpectedResponse.
                new("#all expected to receive an Array from the resource, instead a #{array.class} was received.") unless array.is_a? Array
        @parent = parent
        @array = array.collect { |item| @parent.new(item) }
      end

      def each(&block)
        @array.each &block
      end

      def [](key)
        @array[key]
      end

      def first
        self[0]
      end

      def last
        self[-1]
      end

      def length
        @array.length
      end

      def inspect
        "#<#{self.class.name}:0x%08x record_count: #{self.length}>" % (object_id * 2)
      end

    end
  end
end
