module Arke
  module Resource
    module Model

      extend ActiveSupport::Concern

      included do
        include ActiveModel
        include ActiveModel::Conversion

        extend ActiveModel::Naming

        include Resource::Fields
        include Resource::Associations
        include Resource::Requests
        include Resource::Collections
        include Resource::URL
        include Resource::Persistence
        include Resource::Callbacks
        include Resource::Naming
        include Resource::Finders

        deserializer  { |body| JSON.parse(body) }

        field :id, type: String


      end

      module ClassMethods

        def all(params={})
          body = make_request(:get, {}, params)
          self.collection.new(self, body)
        end

        def find(id, params={})
          body = make_request(:get, {}, {id: id}.merge(params))
          new(body)
        end

      end

      def initialize(attributes={})
        @_attributes = attributes
      end

      def to_param
        self.id
      end

      def inspect
        id = "id: ''#{self.id}'" if try(:id)
        "#<#{self.class.name}:0x%08x>" % (object_id * 2)
      end

    end
  end
end
