module Arke
  module Resource
    module Persistence
      extend ActiveSupport::Concern

      module Callbacks
        extend ActiveSupport::Concern

        included do
          class_attribute :after_destroy_callbacks

          alias_method_chain :destroy, :after_destroy
        end


        module ClassMethods

          def after_destroy(method=nil, &block)
            self.after_destroy_callbacks ||= []
            self.after_destroy_callbacks << method if method
            self.after_destroy_callbacks << block  if block_given?
          end

        end

      end

      included do
        include Callbacks
        include ActiveModel::Validations

      end

      module ClassMethods

        def destroy(id)
          new(id).destroy
        end

        def destroy_all(params={})
          puts 'making delete request'
          make_request(:delete, params)
        end

      end

      def destroy
        delete @_attributes
      end

      def save
        unless self.persisted?
          return true if (@_attributes = post(@_attributes))
        else
          return update(@_attributes)
        end
        false
      end

      def update(attributes)
        @_attributes = attributes
        return true if (@_attributes = put(@_attributes))
        false
      end

      def reload
        @_attributes = self.get(@_attributes.clone)
        self
      end

      def persisted?
        !self.id.empty?
      end

      private

      def destroy_with_after_destroy
        destroy_without_after_destroy
        self.after_destroy_callbacks ||= []
        self.after_destroy_callbacks.each do |callback|
          execute_callback(callback)
        end
      end


    end
  end
end